from qtpy.QtWidgets import QMainWindow, QApplication, QWidget, QVBoxLayout, QSizePolicy, QSizeGrip, QMenu
from qtpy.QtGui import QMouseEvent
from qtpy.QtCore import Qt, QMetaObject, QTimer, Slot

from .titlebar import Titlebar


class FramelessMainWindow(QMainWindow):
    """Frameless main window.

    Args:
        parent (QWidget, optional): Parent widget.
    """

    def __init__(self, parent=None):
        super(FramelessMainWindow, self).__init__(parent)

        self._contentWidgets = []

        self.setWindowFlags(Qt.Window
                            | Qt.FramelessWindowHint
                            | Qt.WindowSystemMenuHint
                            | Qt.WindowCloseButtonHint
                            | Qt.WindowMinimizeButtonHint
                            | Qt.WindowMaximizeButtonHint)

        self.setAttribute(Qt.WA_TranslucentBackground)

        # For preview
        screen = QApplication.desktop().availableGeometry()
        self.resize(screen.width() / 2, screen.height() / 2)
        self.setContentsMargins(0, 0, 0, 0)

        self._centralWidget = QWidget(self)
        self._centralWidget.setObjectName("centralWidget")
        self._centralWidget.setContentsMargins(0, 0, 0, 0)

        self._centralLayout = QVBoxLayout(self._centralWidget)
        self._centralLayout.setAlignment(Qt.AlignBottom)
        self._centralLayout.setSpacing(3)
        self._centralLayout.setContentsMargins(5, 5, 5, 5)

        self._bar = Titlebar(self)
        self._bar.showRestoreButton(False)
        self._bar.showLogoLabel(False)
        self._bar.showLogoButton(True)
        self._centralLayout.addWidget(self._bar)
        self._centralLayout.setAlignment(self._bar, Qt.AlignVCenter)

        self._centralLayout.addSpacing(3)

        self._contentWidget = QWidget(self)
        self._contentWidget.setSizePolicy(
            QSizePolicy.Expanding,
            QSizePolicy.Expanding)
        self._contentWidgetLayout = QVBoxLayout(self._contentWidget)
        self._contentWidgetLayout.setContentsMargins(6, 6, 6, 6)
        self._centralLayout.addWidget(self._contentWidget)

        self._sizegrip = QSizeGrip(self._centralWidget)
        self._centralLayout.addWidget(self._sizegrip, 0, Qt.AlignBottom | Qt.AlignRight)

        self._centralWidget.setLayout(self._centralLayout)
        self.setCentralWidget(self._centralWidget)

        self._fullscreenTitlebarTimer = QTimer(self)
        self._fullscreenTitlebarTimer.setObjectName("_fullscreenTitlebarTimer")

        self._contentWidget.setAutoFillBackground(True)

        self.minimizeClicked = self._bar.minimizeClicked
        self.maximizeClicked = self._bar.maximizeClicked
        self.restoreClicked = self._bar.restoreClicked
        self.closeClicked = self._bar.closeClicked

        QMetaObject.connectSlotsByName(self)

    def showSizeGrip(self, value: bool):
        """Show or hide size grip.

        Args:
            value (bool): To show or to hide.
        """
        self._sizegrip.setVisible(value)

    def setMenu(self, menu: QMenu):
        """Set menu for app icon.

        Args:
            menu (QMenu): QMenu to show.
        """
        self._bar.setMenu(menu)

    def setTitlebarHeight(self, height: int):
        """Set titlebar height.

        Args:
            height (int): Titlebar height.
        """
        self._bar.setTitlebarHeight(height)

    def addContentWidget(self, widget: QWidget):
        """Add master widget to window.

        Args:
            widget (QWidget): Content widget.
        """
        self._contentWidgets.append(widget)
        self._contentWidgetLayout.addWidget(widget)

    def insertContentWidget(self, index, widget: QWidget):
        """Insert master widget to window at pos.

        Args:
            index (int): Index
            widget (QWidget): Content widget.
        """
        self._contentWidgets.insert(index, widget)
        self._contentWidgetLayout.insertWidget(index, widget)

    def showFullScreen(self) -> None:
        """Show app in fullscreen mode"""
        self.setWindowState(Qt.WindowFullScreen)

        firstshow_timer = QTimer(self)
        firstshow_timer.timeout.connect(self._on_showFullScreen)
        firstshow_timer.start(3000)

    def _on_showFullScreen(self):
        if self.windowState() == Qt.WindowFullScreen:
            self._bar.setVisible(False)
            self._separator.setVisible(False)

    def mousePressEvent(self, event: QMouseEvent) -> None:
        """Handle mouse press events

        Args:
            index (QMouseEvent): Mouse press event
        """
        if self.windowState() == Qt.WindowFullScreen:
            ypos = event.globalY()
            if ypos <= 5:
                self._bar.setVisible(True)
                self._separator.setVisible(True)
                self._fullscreenTitlebarTimer.start(2500)
        else:
            self._fullscreenTitlebarTimer.stop()

    @Slot()
    def on__fullscreenTitlebarTimer_timeout(self):
        if self.windowState() == Qt.WindowFullScreen:
            self._bar.setVisible(False)
            self._separator.setVisible(False)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""QRainbowStyle default palette."""

# Standard library imports
from collections import OrderedDict


class BasePalette:
    """Base class for palettes."""

    # buttons background, active tabs name background, titlebar icons background on hover
    COLOR_BACKGROUND_LIGHT = '#505F69'
    # borders, non-active tabs names, headers in tables, titlebar icons background
    COLOR_BACKGROUND_NORMAL = '#32414B'
    # widgets background
    COLOR_BACKGROUND_DARK = '#19232D'

    # text
    COLOR_FOREGROUND_LIGHT = '#F0F0F0'
    # not used?
    COLOR_FOREGROUND_NORMAL = '#AAAAAA'
    # disabled widgets text
    COLOR_FOREGROUND_DARK = '#787878'

    # borders on selection
    COLOR_SELECTION_LIGHT = '#148CD2'
    # checked widgets borders, current tab indicator
    COLOR_SELECTION_NORMAL = '#1464A0'
    # disabled widgets borders, disabled tabs indicators
    COLOR_SELECTION_DARK = '#14506E'

    OPACITY_TOOLTIP = 230

    # Size
    SIZE_BORDER_RADIUS = '4px'

    # Borders
    BORDER_LIGHT = '1px solid $COLOR_BACKGROUND_LIGHT'
    BORDER_NORMAL = '1px solid $COLOR_BACKGROUND_NORMAL'
    BORDER_DARK = '1px solid $COLOR_BACKGROUND_DARK'

    BORDER_SELECTION_LIGHT = '1px solid $COLOR_SELECTION_LIGHT'
    BORDER_SELECTION_NORMAL = '1px solid $COLOR_SELECTION_NORMAL'
    BORDER_SELECTION_DARK = '1px solid $COLOR_SELECTION_DARK'

    # Example of additional widget specific variables
    W_STATUS_BAR_BACKGROUND_COLOR = COLOR_SELECTION_DARK

    # Paths
    PATH_RESOURCES = "':/qss_icons'"

    @classmethod
    def to_dict(cls, colors_only=False):
        """Convert variables to dictionary."""
        order = [
            'COLOR_BACKGROUND_LIGHT',
            'COLOR_BACKGROUND_NORMAL',
            'COLOR_BACKGROUND_DARK',
            'COLOR_FOREGROUND_LIGHT',
            'COLOR_FOREGROUND_NORMAL',
            'COLOR_FOREGROUND_DARK',
            'COLOR_SELECTION_LIGHT',
            'COLOR_SELECTION_NORMAL',
            'COLOR_SELECTION_DARK',
            'OPACITY_TOOLTIP',
            'SIZE_BORDER_RADIUS',
            'BORDER_LIGHT',
            'BORDER_NORMAL',
            'BORDER_DARK',
            'BORDER_SELECTION_LIGHT',
            'BORDER_SELECTION_NORMAL',
            'BORDER_SELECTION_DARK',
            'W_STATUS_BAR_BACKGROUND_COLOR',
            'PATH_RESOURCES',
        ]
        dic = OrderedDict()
        for var in order:
            value = getattr(cls, var)

            if colors_only:
                if not var.startswith('COLOR'):
                    value = None

            if value:
                dic[var] = value

        return dic

    @classmethod
    def color_palette(cls):
        """Return the ordered colored palette dictionary."""
        return cls.to_dict(colors_only=True)


class DarkBlue(BasePalette):
    """Theme variables."""

    # Color
    COLOR_BACKGROUND_LIGHT = '#505F69'
    COLOR_BACKGROUND_NORMAL = '#32414B'
    COLOR_BACKGROUND_DARK = '#19232D'

    COLOR_FOREGROUND_LIGHT = '#F0F0F0'
    COLOR_FOREGROUND_NORMAL = '#AAAAAA'
    COLOR_FOREGROUND_DARK = '#787878'

    COLOR_SELECTION_LIGHT = '#148CD2'
    COLOR_SELECTION_NORMAL = '#1464A0'
    COLOR_SELECTION_DARK = '#14506E'

    W_STATUS_BAR_BACKGROUND_COLOR = COLOR_SELECTION_DARK


class Oceanic(BasePalette):
    """Theme variables."""

    # Color
    COLOR_BACKGROUND_LIGHT = '#314147'
    COLOR_BACKGROUND_NORMAL = '#3B4950'
    COLOR_BACKGROUND_DARK = '#263238'

    COLOR_FOREGROUND_LIGHT = '#C4D6DB'
    COLOR_FOREGROUND_NORMAL = '#56BFBA'
    COLOR_FOREGROUND_DARK = '#56BFBA'

    COLOR_SELECTION_LIGHT = '#097D74'
    COLOR_SELECTION_NORMAL = '#136460'
    COLOR_SELECTION_DARK = '#1B2529'

    W_STATUS_BAR_BACKGROUND_COLOR = COLOR_SELECTION_DARK


class Cyberpunk(BasePalette):
    """Theme variables."""

    # Color
    COLOR_BACKGROUND_LIGHT = '#314147'
    COLOR_BACKGROUND_NORMAL = '#010014'
    COLOR_BACKGROUND_DARK = '#090D0C'

    COLOR_FOREGROUND_LIGHT = '#FBEC08'
    COLOR_FOREGROUND_NORMAL = '#49666e'
    COLOR_FOREGROUND_DARK = '#EC1E77'

    COLOR_SELECTION_LIGHT = '#51BCE2'
    COLOR_SELECTION_NORMAL = '#51BCE2'
    COLOR_SELECTION_DARK = '#51BCE2'

    W_STATUS_BAR_BACKGROUND_COLOR = COLOR_SELECTION_DARK


class DarkOrange(BasePalette):
    """Theme variables."""

    # Color
    COLOR_BACKGROUND_LIGHT = '#505F69'
    COLOR_BACKGROUND_NORMAL = '#32414B'
    COLOR_BACKGROUND_DARK = '#19232D'

    COLOR_FOREGROUND_LIGHT = '#F0F0F0'
    COLOR_FOREGROUND_NORMAL = '#AAAAAA'
    COLOR_FOREGROUND_DARK = '#787878'

    COLOR_SELECTION_LIGHT = '#ffbb54'
    COLOR_SELECTION_NORMAL = '#F9AA33'
    COLOR_SELECTION_DARK = '#fa9e14'

    W_STATUS_BAR_BACKGROUND_COLOR = COLOR_SELECTION_DARK


class LightOrange(BasePalette):
    COLOR_BACKGROUND_LIGHT = '#F0F0F0'
    COLOR_BACKGROUND_NORMAL = '#D1D1D1'
    COLOR_BACKGROUND_DARK = '#DDDDDD'

    COLOR_FOREGROUND_LIGHT = '#000000'
    COLOR_FOREGROUND_NORMAL = '#d2d5d9'
    COLOR_FOREGROUND_DARK = '#d2d5d9'

    COLOR_SELECTION_LIGHT = '#ffbb54'
    COLOR_SELECTION_NORMAL = '#F9AA33'
    COLOR_SELECTION_DARK = '#fa9e14'

    W_STATUS_BAR_BACKGROUND_COLOR = COLOR_SELECTION_DARK



from qtpy.QtWidgets import QToolButton, QSizePolicy
from qtpy.QtGui import QRegion, QIcon
from qtpy.QtCore import QRect, QSize, QEvent


class titleBarButton(QToolButton):

    def __init__(self, icon: QIcon, hovericon: QIcon, parent=None):
        super(titleBarButton, self).__init__(parent)

        self.setMask(QRegion(QRect(0, 0, 21, 21), QRegion.Ellipse))
        self.setMinimumSize(20, 20)
        self.setIconSize(QSize(20, 20))
        self.setAutoFillBackground(True)

        self.icon = icon
        self.hovericon = hovericon

        self.setIcon(self.icon)

    def enterEvent(self, event: QEvent) -> None:
        self.setIcon(self.hovericon)

    def leaveEvent(self, event: QEvent) -> None:
        self.setIcon(self.icon)


class titleBarWindowsButton(QToolButton):

    def __init__(self, icon: QIcon = None, hovericon: QIcon = None, parent=None):
        super(titleBarWindowsButton, self).__init__(parent)

        if not icon:
            raise Exception("Icon is required")

        self.icon = icon
        self.hoverIcon = hovericon

        # iconsize = self.icon.availableSizes()[0]
        iconsize = QSize(45, 30)
        self.setEnabled(True)
        self.setMinimumSize(iconsize)
        self.setAutoFillBackground(True)
        self.setText("")
        self.setIconSize(iconsize)
        self.setChecked(False)

        self.setIcon(self.icon)

        sizepolicy = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        sizepolicy.setHorizontalStretch(0)
        sizepolicy.setVerticalStretch(0)
        sizepolicy.setHeightForWidth(self.sizePolicy().hasHeightForWidth())
        self.setSizePolicy(sizepolicy)

    def getIcons(self):
        return self.icon, self.hoverIcon

    def enterEvent(self, event: QEvent) -> None:
        if self.hoverIcon:
            self.setIcon(self.hoverIcon)

    def leaveEvent(self, event: QEvent) -> None:
        self.setIcon(self.icon)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""QRainbowStyle is a fully customizable stylesheet for Python and Qt applications.

This module provides a function to load pre-compiled stylesheets. To generate your own
style based on custom color palette clone package from project homepage.

qrainbowstyle.windows module adds frameless windows, which can replace original QMainWindow,
QDialog and QMessageBox windows. Frameless windows support resizing, moving, snapping to edges.
You can set global icon and app name for all windows you create in your app. In frameless
main window it is also possible to set custom QMenu to QToolButton with app icon.

First, start importing our module

.. code-block:: python

    import qrainbowstyle

Then you can get stylesheet provided by QRainbowStyle for various Qt wrappers
as shown below

.. code-block:: python

    # PySide2
    stylesheet = qrainbowstyle.load_stylesheet_pyside2(style='darkblue')
    # PyQt5
    stylesheet = qrainbowstyle.load_stylesheet_pyqt5(style='darkblue')

Alternatively, from environment variables provided by QtPy, Qt.Py

.. code-block:: python

    # QtPy
    stylesheet =  qrainbowstyle.load_stylesheet(style='darkblue')
    # Qt.Py
    stylesheet = qrainbowstyle.load_stylesheet(style='darkblue', qt_api=Qt.__binding__)

Finally, set your QApplication with it

.. code-block:: python

    app.setStyleSheet(stylesheet)

To load frameless window in your app import both qrainbowstyle and qrainbowstyle.windows modules

.. code-block:: python

    import qrainbowstyle
    import qrainbowstyle.windows

Initialize qt app and load choosen stylesheet.
Next, create instances of frameless window and your master widget with content you want to show.

.. code-block:: python

    # Create app and load selected stylesheet
    app = QtWidgets.QApplication(sys.argv)
    app.setStyleSheet(qrainbowstyle.load_stylesheet(style="cyberpunk"))

    # Package options
    # qrainbowstyle.align_buttons_left()      # align titlebar buttons to left side
    # qrainbowstyle.use_darwin_buttons()      # use darwin style buttons
    qrainbowstyle.set_app_name("My new application")  # set global name for application
    # qrainbowstyle.set_app_icon("icon.ico")    # set global app icon

    # Create frameless mainwindow
    win = qrainbowstyle.windows.FramelessMainWindow()

    # Create content widget and pass reference to main window
    widget = MasterWidget(win)

    # Add widget to main window and show it
    win.addContentWidget(widget)
    win.show()

    sys.exit(app.exec())

Enjoy!

"""

# Standard library imports
import os
import sys
import logging
import platform
import qrainbowstyle

__version__ = "0.5.4"

_logger = logging.getLogger(__name__)

# Folder's path
REPO_PATH = os.path.dirname(os.path.abspath(os.path.dirname(__file__)))

EXAMPLE_PATH = os.path.join(REPO_PATH, 'example')
IMAGES_PATH = os.path.join(REPO_PATH, 'images')
PACKAGE_PATH = os.path.join(REPO_PATH, 'qrainbowstyle')

QSS_PATH = os.path.join(PACKAGE_PATH, 'qss')
RC_PATH = os.path.join(PACKAGE_PATH, 'rc')
SVG_PATH = os.path.join(PACKAGE_PATH, 'svg')
STYLES_PATH = os.path.join(PACKAGE_PATH, 'styles')

BUTTONS_DARWIN_PATH = os.path.join(SVG_PATH, 'buttons_darwin')
BUTTONS_NT_PATH = os.path.join(SVG_PATH, 'buttons_nt')

# File names
QSS_FILE = 'style.qss'
QRC_FILE = QSS_FILE.replace('.qss', '.qrc')

MAIN_SCSS_FILE = 'main.scss'
STYLES_SCSS_FILE = '_styles.scss'
VARIABLES_SCSS_FILE = '_variables.scss'

# File paths
QSS_FILEPATH = os.path.join(PACKAGE_PATH, QSS_FILE)
QRC_FILEPATH = os.path.join(PACKAGE_PATH, QRC_FILE)

MAIN_SCSS_FILEPATH = os.path.join(QSS_PATH, MAIN_SCSS_FILE)
STYLES_SCSS_FILEPATH = os.path.join(QSS_PATH, STYLES_SCSS_FILE)
VARIABLES_SCSS_FILEPATH = os.path.join(QSS_PATH, VARIABLES_SCSS_FILE)

DEPRECATION_MSG = '''This function will be deprecated in v3.0.
Please, set the wanted binding by using QtPy environment variable QT_API,
then use load_stylesheet() or use load_stylesheet()
passing the argument qt_api='wanted_binding'.'''

# Use lightstyle as default

USE_DARWIN_BUTTONS = False
ALIGN_BUTTONS_LEFT = False

# Global app name
APP_NAME = None

# Path to app icon
APP_ICON_PATH = None


def set_app_name(name: str):
    """Set global app name which will be used in titlebars"""
    qrainbowstyle.APP_NAME = name


def set_app_icon(icon_path: str):
    """Set path to app icon which will be used in titlebars"""
    qrainbowstyle.APP_ICON_PATH = icon_path


def align_buttons_left():
    """Align titlebar buttons to left"""
    qrainbowstyle.ALIGN_BUTTONS_LEFT = True
    _logger.info("Buttons will be aligned to left")


def use_darwin_buttons():
    qrainbowstyle.USE_DARWIN_BUTTONS = True
    _logger.info("Darwin buttons style has been enabled")


def get_available_styles():
    return [x for x in os.listdir(STYLES_PATH) if x != '__init__.py']


def get_current_palette():
    """Returns loaded palette"""
    try:
        from style_rc import palette
        return palette
    except ModuleNotFoundError:
        raise ModuleNotFoundError("Cannot find current palette. Did you load style sheet?")


def _apply_os_patches(palette):
    """
    Apply OS-only specific stylesheet pacthes.

    Returns:
        str: stylesheet string (css).
    """
    os_fix = ""

    if platform.system().lower() == 'darwin':
        # See issue #12
        os_fix = '''
        QDockWidget::title
        {{
            background-color: {color};
            text-align: center;
            height: 12px;
        }}
        '''.format(color=palette.COLOR_BACKGROUND_NORMAL)

    # Only open the QSS file if any patch is needed
    if os_fix:
        _logger.info("Found OS patches to be applied.")

    return os_fix


def _apply_binding_patches():
    """
    Apply binding-only specific stylesheet patches for the same OS.

    Returns:
        str: stylesheet string (css).
    """
    binding_fix = ""

    if binding_fix:
        _logger.info("Found binding patches to be applied.")

    return binding_fix


def _apply_version_patches():
    """
    Apply version-only specific stylesheet patches for the same binding.

    Returns:
        str: stylesheet string (css).
    """
    version_fix = ""

    if version_fix:
        _logger.info("Found version patches to be applied.")

    return version_fix


def _apply_application_patches(palette, QCoreApplication, QPalette, QColor):
    """
    Apply application level fixes on the QPalette.

    The import names args must be passed here because the import is done
    inside the load_stylesheet() function, as QtPy is only imported in
    that moment for setting reasons.
    """
    # See issue #139
    color = palette.COLOR_SELECTION_LIGHT
    qcolor = QColor(color)

    # Todo: check if it is qcoreapplication indeed
    app = QCoreApplication.instance()

    _logger.info("Found application patches to be applied.")

    if app:
        _palette = app.palette()
        _palette.setColor(QPalette.Normal, QPalette.Link, qcolor)
        app.setPalette(_palette)
    else:
        _logger.warning("No QCoreApplication instance found. "
                        "Application patches not applied. "
                        "You have to call load_stylesheet function after "
                        "instantiation of QApplication to take effect. ")


def _load_stylesheet(qt_api='', style=''):
    """
    Load the stylesheet based on QtPy abstraction layer environment variable.

    If the argument is not passed, it uses the current QT_API environment
    variable to make the imports of Qt bindings. If passed, it sets this
    variable then make the imports.

    Args:
        qt_api (str): qt binding name to set QT_API environment variable.
                      Default is ''. Possible values are pyside2,
                      pyqt5. Not case sensitive.

    Note:
        - Note that the variable QT_API is read when first imported. So,
          pay attention to the import order.
        - OS, binding and binding version number, and application specific
          patches are applied in this order.

    Returns:
        str: stylesheet string (css).
    """

    if qt_api:
        os.environ['QT_API'] = qt_api

    # Import is made after setting QT_API
    from qtpy.QtCore import QCoreApplication, QFile, QTextStream
    from qtpy.QtGui import QColor, QPalette

    # Search for style in styles directory
    style_dir = None

    available_styles = [x for x in os.listdir(STYLES_PATH) if x != '__init__.py']
    _logger.debug(f"Available styles: {available_styles}")
    for stl in available_styles:
        if style.lower() in stl.lower():
            style_dir = stl
            break

    if style_dir is None:
        stylesheet = ""
        raise FileNotFoundError("Style " + style + " does not exists")

    # check if any style_rc was loaded before
    if "style_rc" in sys.modules:
        _logger.info("Found already imported style in sys.modules")

        # use qCleanupResources to remove all resource files
        global style_rc
        style_rc.qCleanupResources()

        # remove imported modules
        for x in [module for module in sys.modules if module.startswith("style_rc")]:
            del sys.modules[x]
            del style_rc

        # remove path to previously imported style from sys.path
        for stylepath in [path for path in sys.path if any(style for style in get_available_styles() if style in path)]:
            sys.path.remove(stylepath)
        _logger.debug("Removed all imported styles")

    try:
        _logger.debug("Loading style from directory: " + style_dir)

        # set style directory
        package_dir = os.path.join(STYLES_PATH, style_dir)
        os.chdir(package_dir)

        # append directory to sys.path and import style_rc
        sys.path.append(package_dir)
        import style_rc

        # get palette
        palette = style_rc.palette
    except FileExistsError:
        raise FileNotFoundError("Missing style_rc.py file")

    _logger.info("Style resources imported successfully")

    # Thus, by importing the binary we can access the resources
    package_dir = os.path.basename(PACKAGE_PATH)
    qss_rc_path = ":" + os.path.join(package_dir, QSS_FILE)

    _logger.debug("Reading QSS file in: %s", qss_rc_path)

    # It gets the qss file from compiled style_rc that was import
    # not from the file QSS as we are using resources
    qss_file = QFile(qss_rc_path)

    if qss_file.exists():
        qss_file.open(QFile.ReadOnly | QFile.Text)
        text_stream = QTextStream(qss_file)
        stylesheet = text_stream.readAll()
        _logger.info("QSS file sucessfuly loaded.")
    else:
        stylesheet = ""
        # Todo: check this raise type and add to docs
        raise FileNotFoundError("Unable to find QSS file '{}' "
                                "in resources.".format(qss_rc_path))

    _logger.debug("Checking patches for being applied.")

    # Todo: check execution order for these functions
    # 1. Apply OS specific patches
    stylesheet += _apply_os_patches(palette)

    # 2. Apply binding specific patches
    stylesheet += _apply_binding_patches()

    # 3. Apply binding version specific patches
    stylesheet += _apply_version_patches()

    # 4. Apply palette fix. See issue #139
    _apply_application_patches(palette, QCoreApplication, QPalette, QColor)

    return stylesheet


def load_stylesheet(qt_api="", style='darkblue'):
    """
    Load the stylesheet. Takes care of importing the rc module.

    Args:
        qt_api (str): Qt binding name to set QT_API environment variable.
                      Default is '', i.e PyQt5 the default QtPy binding.
                      Possible values are pyside2, pyqt5.
                      Not case sensitive.

        style (str): Style to use. Default is 'darkblue'

    Returns:
        str: the stylesheet string.
    """

    stylesheet = ""

    if qt_api:
        stylesheet = _load_stylesheet(qt_api=qt_api, style=style)

    else:
        stylesheet = _load_stylesheet(qt_api='pyqt5', style=style)

    return stylesheet


def load_stylesheet_pyside2(style='darkblue'):
    """
    Load the stylesheet for use in a PySide2 application.

    Returns:
        str: the stylesheet string.
    """
    return _load_stylesheet(qt_api='pyside2', style=style)


def load_stylesheet_pyqt5(style='darkblue'):
    """
    Load the stylesheet for use in a PyQt5 application.

    Returns:
        str: the stylesheet string.
    """
    return _load_stylesheet(qt_api='pyqt5', style=style)



from qtpy.QtWidgets import QToolButton, QLabel
from qtpy.QtGui import QPixmap, QIcon
from qtpy.QtCore import QSize, Qt

import qrainbowstyle


class appLogoButton(QToolButton):
    """
        Clickable main window button with app logo.
        Menu can be added using setMenu().
    """

    def __init__(self, parent):
        super(appLogoButton, self).__init__(parent)

        self.setIcon(QIcon(qrainbowstyle.APP_ICON_PATH))

        self.setFixedSize(32, 32)
        self.setStyleSheet("border: none;")
        self.setIconSize(QSize(28, 28))
        self.setArrowType(Qt.NoArrow)
        self.setPopupMode(QToolButton.InstantPopup)


class appLogoLabel(QLabel):
    """
        Label with app logo. Is show in FramelessDialog
        and FramelessMessageBox.
    """

    def __init__(self, parent):
        super(appLogoLabel, self).__init__(parent)

        self.setPixmap(QPixmap(qrainbowstyle.APP_ICON_PATH))
        self.setScaledContents(True)
        self.setFixedSize(25, 25)
        self.setStyleSheet("border: none;")



from qtpy.QtWidgets import QWidget, QSizePolicy, QHBoxLayout
from qtpy.QtGui import QIcon
from qtpy.QtCore import Qt

from .windowButtons import titleBarWindowsButton, titleBarButton

import qrainbowstyle


def getButtons(parent):
    """
        Loads titlebar buttons to dict depending on selected style (NT/Darwin)
    Args:
        parent (QWidget, required): Widget where buttons will be used.

    Returns:
        buttons (dict): Dict with close, minimize, restore and maximize buttons.
    """

    if qrainbowstyle.USE_DARWIN_BUTTONS:
        return {"minimize": titleBarButton(QIcon(":/qss_icons/rc/button_darwin_minimize.png"),
                                           QIcon(":/qss_icons/rc/button_darwin_minimize_hover.png"),
                                           parent),
                "maximize": titleBarButton(QIcon(":/qss_icons/rc/button_darwin_maximize.png"),
                                           QIcon(":/qss_icons/rc/button_darwin_maximize_hover.png"),
                                           parent),
                "restore": titleBarButton(QIcon(":/qss_icons/rc/button_darwin_restore_hover.png"),
                                          QIcon(":/qss_icons/rc/button_darwin_restore_hover.png"),
                                          parent),
                "close": titleBarButton(QIcon(":/qss_icons/rc/button_darwin_close.png"),
                                        QIcon(":/qss_icons/rc/button_darwin_close_hover.png"),
                                        parent)}

    else:
        return {"minimize": titleBarWindowsButton(QIcon(":/qss_icons/rc/button_nt_minimize.png"),
                                                  QIcon(":/qss_icons/rc/button_nt_minimize_hover.png"),
                                                  parent),
                "maximize": titleBarWindowsButton(QIcon(":/qss_icons/rc/button_nt_maximize.png"),
                                                  QIcon(":/qss_icons/rc/button_nt_maximize_hover.png"),
                                                  parent),
                "restore": titleBarWindowsButton(QIcon(":/qss_icons/rc/button_nt_restore.png"),
                                                 QIcon(":/qss_icons/rc/button_nt_restore_hover.png"),
                                                 parent),
                "close": titleBarWindowsButton(QIcon(":/qss_icons/rc/button_nt_close.png"),
                                               QIcon(":/qss_icons/rc/button_nt_close_hover_red.png"),
                                               parent)}


class buttonsWidget(QWidget):
    """Widget which contains titlebar buttons"""

    def __init__(self, parent):
        super(buttonsWidget, self).__init__(parent)

        self.setObjectName("buttonsWidget")
        sizepolicy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Preferred)
        sizepolicy.setHorizontalStretch(0)
        sizepolicy.setVerticalStretch(0)
        sizepolicy.setHeightForWidth(self.sizePolicy().hasHeightForWidth())
        self.setSizePolicy(sizepolicy)
        self.setStyleSheet("padding: 0px;")

        self.buttonsLayout = QHBoxLayout(self)
        self.buttonsLayout.setContentsMargins(0, 0, 0, 0)
        self.buttonsLayout.setSpacing(0)
        self.buttonsLayout.setAlignment(Qt.AlignVCenter)
        self.setLayout(self.buttonsLayout)

        btns = getButtons(self)

        self.btnMinimize = btns["minimize"]
        self.btnMaximize = btns["maximize"]
        self.btnRestore = btns["restore"]
        self.btnClose = btns["close"]

        if qrainbowstyle.ALIGN_BUTTONS_LEFT:
            # reversed order
            self.buttonsLayout.addWidget(self.btnClose)
            if not qrainbowstyle.USE_DARWIN_BUTTONS:
                self.buttonsLayout.addWidget(self.btnRestore)
            self.buttonsLayout.addWidget(self.btnMaximize)
            self.buttonsLayout.addWidget(self.btnMinimize)
        else:
            # normal order
            self.buttonsLayout.addWidget(self.btnMinimize)
            self.buttonsLayout.addWidget(self.btnMaximize)
            if not qrainbowstyle.USE_DARWIN_BUTTONS:
                self.buttonsLayout.addWidget(self.btnRestore)
            self.buttonsLayout.addWidget(self.btnClose)

        self.btnMinimize.setObjectName("btnMinimize")
        self.btnMaximize.setObjectName("btnMaximize")
        if not qrainbowstyle.USE_DARWIN_BUTTONS:
            self.btnRestore.setObjectName("btnRestore")
        self.btnClose.setObjectName("btnClose")

        if qrainbowstyle.USE_DARWIN_BUTTONS:
            self.buttonsLayout.setSpacing(8)



from qtpy.QtWidgets import QApplication, QFrame, QMenu, QLabel, QHBoxLayout, QWidget
from qtpy.QtCore import Signal, QPoint, QMetaObject, QRect, Slot, Qt
from qtpy.QtGui import QPalette

from .appLogoButton import appLogoLabel, appLogoButton
from .buttonsWidget import buttonsWidget

import qrainbowstyle


def getWorkspace() -> QRect:
    """Returns workspace area"""
    return QApplication.desktop().availableGeometry()


class Titlebar(QFrame):
    minimizeClicked = Signal()
    maximizeClicked = Signal()
    restoreClicked = Signal()
    closeClicked = Signal()
    """Titlebar documentation"""

    def __init__(self, parentwindow: QWidget, parent=None):
        super(Titlebar, self).__init__(parent)
        self.setObjectName("titlebar")

        self.window = parentwindow
        self.offset = QPoint()
        self.maxNormal = False
        self.moving = False

        self.setAutoFillBackground(True)
        self.setFixedHeight(45)
        self.setContentsMargins(0, 0, 0, 0)
        self.setBackgroundRole(QPalette.Highlight)

        self.layout = QHBoxLayout(self)
        self.layout.setAlignment(Qt.AlignVCenter)
        self.layout.setAlignment(Qt.AlignLeft)
        self.layout.setSpacing(2)
        self.layout.setContentsMargins(5, 0, 5, 0)
        self.setLayout(self.layout)

        self.appLogoLabel = appLogoLabel(self)
        self.layout.addWidget(self.appLogoLabel)

        self.appLogoButton = appLogoButton(self)
        self.layout.addWidget(self.appLogoButton)

        self.label = QLabel(self)
        self.label.setText(qrainbowstyle.APP_NAME)
        self.label.setAlignment(Qt.AlignVCenter)
        self.label.mouseMoveEvent = self.mouseMoveEvent
        self.label.mousePressEvent = self.mousePressEvent
        self.label.mouseReleaseEvent = self.mouseReleaseEvent
        self.layout.addWidget(self.label)
        if qrainbowstyle.ALIGN_BUTTONS_LEFT:
            self.layout.setAlignment(self.label, Qt.AlignCenter)

        self.layout.insertStretch(3)

        self.buttonsWidget = buttonsWidget(self)

        if qrainbowstyle.ALIGN_BUTTONS_LEFT:
            self.layout.insertWidget(0, self.buttonsWidget)
        else:
            self.layout.addWidget(self.buttonsWidget)

        # auto connect signals
        QMetaObject.connectSlotsByName(self)

    # connecting buttons signals
    @Slot()
    def on_btnClose_clicked(self):
        self.window.close()
        self.closeClicked.emit()

    @Slot()
    def on_btnRestore_clicked(self):
        self.showRestoreButton(False)
        self.showMaximizeButton(True)
        self.window.setWindowState(Qt.WindowNoState)
        self.maxNormal = False
        self.restoreClicked.emit()

    @Slot()
    def on_btnMaximize_clicked(self):
        if qrainbowstyle.USE_DARWIN_BUTTONS:
            if self.window.windowState() == Qt.WindowMaximized:
                self.window.setWindowState(Qt.WindowNoState)
                self.maxNormal = False
            else:
                self.window.setWindowState(Qt.WindowMaximized)
                self.maxNormal = True
        else:
            self.showRestoreButton(True)
            self.showMaximizeButton(False)
            self.window.setWindowState(Qt.WindowMaximized)
            self.maxNormal = True
        self.maximizeClicked.emit()

    @Slot()
    def on_btnMinimize_clicked(self):
        self.window.showMinimized()
        self.minimizeClicked.emit()

    def setWindowTitle(self, title: str) -> None:
        """Set window title"""
        self.label.setText(title)

    def showLogoLabel(self, value: bool):
        """Show or hide app logo label"""
        self.appLogoLabel.setVisible(value)

    def showLogoButton(self, value: bool):
        """Show or hide menu button with app logo button"""
        self.appLogoButton.setVisible(value)

    def showRestoreButton(self, value):
        if not qrainbowstyle.USE_DARWIN_BUTTONS:
            self.buttonsWidget.btnRestore.setVisible(value)

    def showMaximizeButton(self, value):
        if not qrainbowstyle.USE_DARWIN_BUTTONS:
            self.buttonsWidget.btnMaximize.setVisible(value)

    def setMenu(self, menu: QMenu):
        """Set app logo button menu"""
        self.appLogoButton.setMenu(menu)

    def setTitlebarHeight(self, height: int):
        self.setFixedHeight(height)

    def showMaxRestore(self):
        """btnMaximize if normal, restore if btnMaximized"""
        if self.maxNormal:
            self.showRestoreButton(False)
            self.showMaximizeButton(True)
            self.window.setWindowState(Qt.WindowNoState)
            self.maxNormal = False

        else:
            self.showRestoreButton(True)
            self.showMaximizeButton(False)
            self.window.setWindowState(Qt.WindowMaximized)
            self.maxNormal = True

    def mousePressEvent(self, event):
        """Handle mouse press events"""
        if event.button() == Qt.LeftButton:
            self.moving = True
            if self.label.underMouse():
                self.offset = event.pos() + self.pos() + self.label.pos()
            else:
                self.offset = event.pos() + self.pos()

    def mouseReleaseEvent(self, event):
        """Handle mouse release events"""
        if self.moving:
            screen = QApplication.desktop().availableGeometry()

            if event.globalY() == 0:
                self.on_btnMaximize_clicked()

            elif event.globalX() == 0:
                self.window.move(0, 0)
                self.window.resize(screen.width() / 2, screen.height())
                self.window.setWindowState(Qt.WindowNoState)
                self.maxNormal = False

            elif event.globalX() + 1 >= screen.width():
                self.window.move(screen.width() / 2, 0)
                self.window.resize(screen.width() / 2, screen.height())
                self.window.setWindowState(Qt.WindowNoState)
                self.maxNormal = False

        self.moving = False

    def mouseDoubleClickEvent(self, event):
        self.showMaxRestore()

    def mouseMoveEvent(self, event):
        """Handle mouse moving"""
        if self.maxNormal:
            workarea = getWorkspace()
            self.moving = False
            self.on_btnRestore_clicked()
            self.window.resize(workarea.width(), workarea.height())
            self.moving = True

        if self.moving:
            if not self.buttonsWidget.underMouse():
                self.window.move(event.globalPos() - self.offset)

    def resizeEvent(self, event):
        """Handle resizing events"""
        self.buttonsWidget.btnMaximize.leaveEvent(None)
        if not qrainbowstyle.USE_DARWIN_BUTTONS:
            self.buttonsWidget.btnRestore.leaveEvent(None)



from qtpy import QtCore, QtWidgets

import logging
import traceback

_logger = logging.getLogger(__name__)


def getWorkspace() -> QtCore.QRect:
    """Returns workspace area"""
    return QtWidgets.QApplication.desktop().availableGeometry()


class OutputLogger(object):
    """Logging cocfiguration class"""

    def __init__(self):
        super(OutputLogger, self).__init__()
        logging.basicConfig(
            level=logging.NOTSET,
            format="%(asctime)s [%(threadName)s] [%(name)s] [%(levelname)s] %(message)s",
            handlers=[
                logging.StreamHandler()
            ])
        _logger.debug("Logger enabled.")


def qt_message_handler(mode, context, message):
    """Qt errors handler"""
    if mode == QtCore.QtInfoMsg:
        mode = 20
    elif mode == QtCore.QtWarningMsg:
        mode = 30
    elif mode == QtCore.QtCriticalMsg:
        mode = 40
    elif mode == QtCore.QtFatalMsg:
        mode = 50
    else:
        mode = 20
    _logger.log(mode, "%s (%s:%d, %s)" % (message, context.file, context.line, context.file))


def catch_exceptions(t, val, tb):
    """Replaces sys.excepthook to catch and log warnings and errors"""
    trace = "".join(traceback.format_exception(t, val, tb))
    _logger.critical(trace)



#!python
# -*- coding: utf-8 -*-
"""Test the qtsass is compiling the SCSS files to QSS."""

# Standard library imports
import tempfile

# Local imports
from qrainbowstyle.utils.scss import create_custom_qss, create_qss


def test_create_qss():
    # Should not raise a CompileError
    create_qss()


def test_create_custom_qss():
    # Should not raise a CompileError
    qss = create_custom_qss(
        'MyAwesomePalette',
        tempfile.mkdtemp(),
        '#ff0000',
        '#cc0000',
        '#aa0000',
        '#00ff00',
        '#00cc00',
        '#00aa00',
        '#0000ff',
        '#0000cc',
        '#0000aa',
        '0px',
    )
    assert qss



from qtpy.QtCore import Signal, Qt
from qtpy.QtWidgets import QDialog, QVBoxLayout

from .titlebar import Titlebar


class FramelessDialog(QDialog):
    closed = Signal()
    """FramelessDialog documentation"""

    def __init__(self, parent=None):
        super(FramelessDialog, self).__init__(parent)
        self.resize(300, 500)

        self.setWindowFlags(Qt.FramelessWindowHint)

        self.layout = QVBoxLayout(self)
        self.layout.setAlignment(Qt.AlignTop)
        self.layout.setContentsMargins(11, 5, 11, 11)
        self.bar = Titlebar(self)
        self.layout.addWidget(self.bar)

        self.closed = self.bar.terminate.clicked
        self.closed.connect(self.close)



from qtpy.QtCore import Qt
from qtpy.QtWidgets import (QGridLayout, QLabel, QStyle, QDialogButtonBox, QSizePolicy)

from .framelessdialog import FramelessDialog


class FramelessMessageBox(FramelessDialog):
    """FramelessMessageBox documentation"""

    def __init__(self, parent=None):
        super(FramelessMessageBox, self).__init__(parent)
        self.resize(300, 100)

        self.grid = QGridLayout(self)
        self.grid.setContentsMargins(0, 0, 0, 0)
        self.grid.setVerticalSpacing(8)
        self.grid.setHorizontalSpacing(0)

        self.iconLabel = QLabel(self)
        self.iconLabel.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        self.iconLabel.setScaledContents(True)
        self.setIcon(QStyle.SP_MessageBoxInformation)
        self.grid.addWidget(self.iconLabel, 0, 0, 2, 1, Qt.AlignTop | Qt.AlignLeft)

        self.label = QLabel(self)
        self.label.setWordWrap(True)
        self.label.setText("QMessageBox Dialog window text. Lorem ipsum mare guer sla ikure mier purso.")
        self.label.setContentsMargins(2, 0, 0, 0)
        self.label.setIndent(9)

        self.grid.addWidget(self.label, 0, 1, 1, 1)

        self.grid.setRowStretch(1, 100)
        self.grid.setRowMinimumHeight(2, 6)

        self.buttonBox = QDialogButtonBox(self)
        self.buttonBox.setStandardButtons(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
        self.grid.addWidget(self.buttonBox, 3, 1, 1, 1)

        self.layout.addLayout(self.grid)
        self.showWindowIcon(False)

    def setText(self, text: str):
        self.label.setText(text)
